#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>
#include <unistd.h>

#include "ulaw.h"

#define FRACTION_OF_SECOND 1/10
#define MAXSAMPLE 65536
#define FREQUENCY 8000

#define MICROS (1000000*FRACTION_OF_SECOND)
#define TICKS (FREQUENCY*FRACTION_OF_SECOND)
unsigned char sample[MAXSAMPLE];
unsigned char outbytes[TICKS];
int flag=0,ticks,counter=0,amp,i,gc=0;

/* max. notes played at same time */
#define MAXNOTES 99
float reg[MAXNOTES]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},samlen,
  regp[MAXNOTES];
int maxnote=0;

FILE *f,*infile;
fd_set rfds,wfds;
struct timeval tv;
int retval,fd_in,fd_out,maxfd;

/* PITCH stuff */
#define TON(i) pow(2.0,((float)(i))/12)
#define VEL(i) 0
#define PITCH_TYPE float
#define PITCH_NEED_INIT
#include "mypitch.h"

void init()
{
  char line[1024];
  /* Pitch init */
  PITCH_INIT;
  /* Sample init */
  sample[0]=128;
  for(counter=1;!feof(f)&&(counter<MAXSAMPLE);counter++)
    sample[counter]=ulaw_dsp[fgetc(f)];
  if(counter>=MAXSAMPLE) {
    fprintf(stderr,"Sample too big. Recompile!\n"); exit(-1); }
  samlen=(float)counter-1;
  fclose(f);

  maxfd=fd_in=fileno(infile);
  if((fd_out=fileno(f))>maxfd) maxfd=fd_out;
  maxfd++;

  setbuf(infile,NULL); setbuf(f,NULL);
}

#define REVERB 0
#define NOTEOFF(i)
#define NOTEON(i) \
  if(maxnote<MAXNOTES) {\
    regp[maxnote]=pitch[(i*2)]; reg[maxnote++]=1.0; }

#define WAIT wait()
void wait(void) 
{
  do {
    for(ticks=0;ticks<TICKS;ticks++) {
      for(counter=0,amp=0;counter<maxnote;counter++) {
	amp+=sample[(int)reg[counter]]-128;
	reg[counter]+=regp[counter];
	if(reg[counter]>=samlen) {
	  reg[counter]=reg[--maxnote]; regp[counter--]=regp[maxnote]; } }
      amp+=128; if(amp>255) amp=255; else if(amp<0) amp=0;
      outbytes[ticks]=(maxnote>0)?dsp_ulaw[amp]:dsp_ulaw[128]; }
    fwrite(outbytes,1,TICKS,f);
    FD_ZERO(&rfds); FD_ZERO(&wfds); tv.tv_sec=tv.tv_usec=0;
    if(maxnote>0) FD_SET(fd_out,&wfds);
    if(flag==0) FD_SET(fd_in, &rfds);
    else if(maxnote==0)
      { fprintf(stderr,"Input ended\n"); exit(0); }
    retval=select(maxfd, &rfds, &wfds, NULL, &tv);
  } while((maxnote>0)&&
	  FD_ISSET(fd_out,&wfds)&&!FD_ISSET(fd_in,&rfds));
}

int
main(int argc,char **argv)
{
  char string[1024];


  if((argc>1)&&strcmp(argv[1],"-"))
    assert(infile=fopen(argv[1],"r"));
  else infile=stdin;

  strcpy(string,argv[0]);
  for(i=strlen(string)-1;(i>=0)&&(string[i]!='/');i--);
  strcpy(&(string[i+1]),"leier_sample.au");
  assert((f=fopen(string,"r")));
  init();

  /* Sound device init */
  if(argc>2) assert((f=fopen(argv[2],"w")));
  else assert((f=fopen("/dev/audio","w")));


#include "my.h"

  fclose(f);
  exit(0);
}
